/************************************************************************
 *   Solid Internet Relay Chat Daemon, src/cloak.c
 *   Copyright (C) 2002-2003 Juan Baez
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* $Id: cloak.c,v 1.35.2.3 2004/07/30 01:31:19 tux316 Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "dynconf.h"
#include "sys.h"
#include "struct.h"
#include "numeric.h"
#include "h.h"
#include "common.h"

/* This code was from Elite IRCd. */

#define B_BASE                  1000

int Maskchecksum (char *data, int len)
{
        int                     i;
        int                     j;

        j=0;
        for (i=0 ; i<len ; i++)
        {
          j += *data++ * (i < 16 ? (i+1)*(i+1) : i*(i-15));

        }

        return (j+B_BASE)%0xffff;
}

/* make_virthost()
 * If the user has a true resolving host, then we mask it the good old elite way.
 * Else if its a non resolving IP to our eyes,  I shall we a host for it:
 *		host-'IP'.c'csum'.usr.americhat.net
 *
 *	Example:
 *		IP: 192.168.1.101
 *		New Host: host-192-168-1-101.c41011.usr.americhat.net 
 *		Masked Host: AC-33115.c41011.usr.americhat.net 
 *
 */
void make_virthost (aClient *sptr, int f)
{
	static char mask[128];
	static char ipmask[64];
	char *s, *dot;
	int csum,i,isdns,e;
	
	/* Sanity check - Tux` */

	s=sptr->sockhost;

	memset(mask, 0, 128);

	csum = Maskchecksum (s, strlen(s));

	if (strlen (s) > 127)
		s[128] = 0;
	
	isdns = 0;
	for (i=0,e=0; i<strlen(s) ; i++)
	{
		if (isalpha(s[i])) 
		{
			isdns = 1;
			break;
		}
	}
	if (isdns)
	{
		dot = strchr (s, '.');
		if (dot==NULL)
			ircsprintf (mask, "%s-%d.%s",HOSTMASK,csum,s);
		else
			ircsprintf (mask, "%s-%d.%s",HOSTMASK,csum,dot+1);
	}
	else 
	{
		strncpy(ipmask, s, sizeof(ipmask));
		ipmask[sizeof(ipmask)-1]='\0';
		while((dot = strrchr(ipmask, '.')))
			*dot = '-';            
		ircsprintf (mask, "%s-%d.c%d.usr.%s", HOSTMASK, csum,
			Maskchecksum(ipmask, strlen(ipmask)), DomainName);
	}
	strncpy (sptr->user->host, mask, HOSTLEN + 1);

}


void x_sethostmask(aClient *sptr) {
	if(IsSAdmin(sptr) && !IsUmodeH(sptr)) {
		strncpyzt(sptr->user->host,SAdmin_host, HOSTLEN + 1);
		sendto_serv_butone(NULL, ":%s SVHOST %s %s", me.name,sptr->name, sptr->user->host);
	} else if (IsAdmin(sptr) && !IsUmodeH(sptr)) {
		strncpyzt(sptr->user->host,Admin_host, HOSTLEN + 1);
		sendto_serv_butone(NULL, ":%s SVHOST %s %s", me.name, sptr->name, sptr->user->host);
	} else if (IsAnOper(sptr) && !IsUmodeH(sptr)) {
		strncpyzt(sptr->user->host,IRCop_host, HOSTLEN + 1);
		sendto_serv_butone(NULL, ":%s SVHOST %s %s", me.name, sptr->name, sptr->user->host);
	} else if (IsLocOp(sptr) && !IsUmodeH(sptr)) {
		strncpyzt(sptr->user->host,IRCop_host, HOSTLEN + 1);
		sendto_serv_butone(NULL, ":%s SVHOST %s %s", me.name,sptr->name, sptr->user->host);
	} else {
		make_virthost(sptr,0);
		if (BadPtr(sptr->user->host))
			make_virthost(sptr,0);
		/* Sanity check. */
		if (BadPtr(sptr->user->host)) {
			sendto_serv_butone(NULL, ":%s KILL %s :"
				"Erroneous Host", me.name, sptr->name);
		}
		sendto_serv_butone(NULL, ":%s SVHOST %s %s", me.name,sptr->name, sptr->user->host);
	}
	if (!IsMasked(sptr)) SetMasked(sptr);
	
}
/* This should be used  by servers only! */
int m_svhost(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;

	if (check_registered (sptr))
		return 0;

	if (!IsServer(sptr))
		return 0;

	if ((acptr=find_client(parv[1], NULL))==NULL)
		return 0;

	if (!IsMasked(acptr))
		SetMasked(acptr);

	ircsprintf (acptr->user->host, "%s", parv[2]);
	if (BadPtr(acptr->user->host)) {
        	sendto_serv_butone(NULL, ":%s KILL %s :"
                	"Erroneous Host", me.name, acptr->name);
		return 0;
	}
	sendto_serv_butone (cptr, ":%s SVHOST %s %s", parv[0], 
		parv[1], parv[2]);
	return 0;
}
/*
 * m_sethost()
 *  parv[0] = sender
 *  parv[1] = newhost
 */
int m_sethost(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    char *p;
    int bad_dns, dots;

    if (check_registered(sptr))
		return 0;

    if (!IsAnOper(sptr))
    {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
    }

    if (parc != 2)
		return 0;

    if (parv[1] == NULL)
	{
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "SETHOST");
		return 0;
    }
	if (strlen(parv[1]) < 4) 
	{
		sendto_one(sptr, ":%s NOTICE %s :*** Notice -- Hostnames must contain a minimum of 4 characters.", me.name, parv[0]);
		return 0;
    }
    if (strlen(parv[1]) > (HOSTLEN - 1)) {
		sendto_one(sptr, ":%s NOTICE %s :*** Notice -- Hostnames are limited to %i characters.", me.name, parv[0], (HOSTLEN - 1));
		return 0;
    }

    dots = 0;
    p = parv[1];
    bad_dns = NO;

    while (*p)
    {
		if (!isalnum(*p))
		{
			if ((*p != '-') && (*p != '.'))
			bad_dns = YES;
		}
		if (*p == '.')
			dots++;
			p++;
		}
		if (bad_dns == YES)
		{
			sendto_one(sptr, ":%s NOTICE %s :*** Notice -- A hostname may contain a-z, 0-9, '-' & '.' - please use only these.", me.name, parv[0]);
			return 0;
		}
		if (!dots)
		{
			sendto_one(sptr, ":%s NOTICE %s :*** Notice -- You must have at least one '.' (dot) in the hostname.", me.name, parv[0]);
			return 0;
		}
		strncpyzt(sptr->user->host, parv[1], HOSTLEN);
		if(!IsMasked(sptr))
			SetMasked(sptr);
		sendto_serv_butone(NULL, ":%s SVHOST %s %s", me.name, parv[0], parv[1]);
		sendto_one(sptr, ":%s NOTICE %s :*** Notice -- Your virtual hostname is now [%s]", me.name, parv[0], parv[1]);
    return 0;
}

