/************************************************************************
 *   Solid Internet Relay Chat Daemon, src/m_gline.c
 *   Copyright (C) 2004 Juan Baez
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* $Id: m_gline.c,v 1.2.2.4 2004/07/30 01:31:19 tux316 Exp $ */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "userban.h"
#include "h.h"
#if defined( HAVE_STRING_H )
#include <string.h>
#else
#include <strings.h>
#endif

extern char *smalldate(time_t);	/* defined in s_misc.c */
extern int  myisnumber(char *);
extern char *mycluster(char *hostname);

int m_gline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    struct userBan *ban, *oban;
    char        buffer[1024];
    char       *user, *host;
    char       *reason;
    char       *current_date;
    aClient    *acptr;
    char        tempuser[USERLEN + 2];
    char        temphost[HOSTLEN + 1];
    int         temporary_gline_time = 0;	/* -Dianora */
    time_t      temporary_gline_time_seconds = 0;
    int         time_specified = 0;
    char       *argv;
    int         i;
    char       fbuf[512];

    if (!IsAnOper(sptr) && !IsServer(sptr)) 
    {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
    }

    if (parc < 2) 
    {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS),
			me.name, parv[0], "GLINE");
		return 0;
    }

    argv = parv[1];

    if ((temporary_gline_time = myisnumber(argv)) >= 0) 
    {
		if (parc < 3) 
		{
			sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS),
			   me.name, parv[0], "GLINE");
			return 0;
		}
		if (temporary_gline_time > 1080000)
			temporary_gline_time = 1080000;

		temporary_gline_time_seconds = (time_t) temporary_gline_time;

		argv = parv[2];
		parc--;
		time_specified = 1;
    }
    else
    {
		temporary_gline_time = 0; 
    }
    
    if(strchr(argv, ' '))
    {
		sendto_one(sptr,
			":%s NOTICE %s :Poorly formatted hostname "
			"(contains spaces). Be sure you are using the form: "
			"/quote GLINE [time] <user@host/nick> :<reason>",
			me.name, parv[0]);
		return 0;
    }
    

    if ((host = strchr(argv, '@')) || *argv == '*') 
    {

		if (host) 		
		{
			user = argv;	
			*(host++) = '\0';
		}
		else 
		{
			user = "*";	
			host = argv;
		}
	
		if (!*host)
			host = "*";

		strncpyzt(tempuser, user, USERLEN + 2);
		strncpyzt(temphost, host, HOSTLEN);
		user = tempuser;
		host = temphost;
    }
    else 
    {
	
		if (!(acptr = find_chasing(sptr, argv, NULL)))
			return 0;

		if (!acptr->user)
			return 0;

		if (IsServer(acptr)) 
		{
			sendto_one(sptr, ":%s NOTICE %s :Can't GLINE a server, use @'s "
		       "where appropriate", me.name, parv[0]);
			return 0;
		}
		tempuser[0] = '*';
		if (*acptr->user->username == '~')
			strcpy(tempuser + 1, (char *) acptr->user->username + 1);
		else
			strcpy(tempuser + 1, acptr->user->username);
		user = tempuser;
		host = mycluster(acptr->user->host);
    }

    if (time_specified)
		argv = parv[3];
    else
		argv = parv[2];

/* FIXME: Shouldnt we use our hown DEFAULT_GLINE_TIME?  - Tux`*/
#ifdef DEFAULT_KLINE_TIME
    if (time_specified <= 0)
    {
		temporary_gline_time = 30;
		temporary_gline_time_seconds = (time_t) temporary_gline_time *(time_t) 60;
    }
#endif
    if (parc > 2) 
    {
		if (*argv)
			reason = argv;
		else
			reason = "No reason";
    }
    else
		reason = "No reason";

    if (!match(user, "akjhfkahfasfjd") &&
		!match(host, "ldksjfl.kss...kdjfd.jfklsjf")) 
    {
		sendto_one(sptr, ":%s NOTICE %s :Can't G-Line *@*", me.name,
		   parv[0]);
		return 0;
    }

    if (temporary_gline_time == 0 && strchr(reason, ':')) 
    {
		sendto_one(sptr,
		   ":%s NOTICE %s :Invalid character ':' in comment",
		   me.name, parv[0]);
		return 0;
    }

    if (temporary_gline_time == 0 && strchr(reason, '#')) 
    {
		sendto_one(sptr,":%s NOTICE %s :Invalid character '#' in comment",
		   me.name, parv[0]);
		return 0;
    }

    ban = make_hostbased_ban(user, host);
    if(!ban)
    {
		sendto_one(sptr, ":%s NOTICE %s :Malformed ban %s@%s", me.name, parv[0],
		   user, host);
		return 0;
    }

    if ((oban = find_userban_exact(ban, 0)))
    {
		if (oban->flags & UBAN_GLINE)
		{
			if (MyClient(sptr))
			{
				sendto_one(sptr, ":%s NOTICE %s :[%s@%s] already %s for %s",
					me.name, parv[0], user, host, NETWORK_GLINNED_NAME,
					oban->reason ? oban->reason : "<No Reason>");
			}
			else
			{
				/* The G-Line does not exist, and sptr is not our client.
				 * However, we should still send it... - Tux` */
				host = get_userban_host(ban, fbuf, 512);
				sendto_serv_butone(sptr, ":%s GLINE %d %s@%s :%s",
					parv[0], temporary_gline_time_seconds, user, host, reason);
				
			}
			userban_free(ban);
			return 0;
		}
    }

    current_date = smalldate((time_t) 0);
	ban->flags |= temporary_gline_time ? (UBAN_GLINE|UBAN_TEMPORARY) : (UBAN_GLINE);
    if (IsServer(sptr))
		ircsprintf(buffer, "%s", reason);
	ircsprintf(buffer, "%s (%s)", reason, current_date);
    ban->reason = (char *) MyMalloc(strlen(buffer) + 1);
    strcpy(ban->reason, buffer);
    
	ban->timeset = timeofday;
	ban->duration = temporary_gline_time_seconds;

    if(user_match_ban(sptr, ban))
    {
		sendto_one(sptr, ":%s NOTICE %s :You attempted to add a gline [%s@%s] "
						 "which would affect yourself. Aborted.",
					me.name, parv[0], user, host);
		userban_free(ban);
		return 0;
    }

    add_hostbased_userban(ban);

    /* Check local users against it */
    for (i = 0; i <= highest_fd; i++)
    {
        if (!(acptr = local[i]) || IsMe(acptr) || IsLog(acptr))
            continue;

        if (IsPerson(acptr) && user_match_ban(acptr, ban))
        {
            sendto_ops(NETWORK_GLINE_NAME" active for %s",
                       get_client_name(acptr, FALSE));
			sendto_serv_butone(sptr, ":%s GLOBOPS :%s active for %s", 
				NETWORK_GLINE_NAME, me.name, get_client_name(acptr, FALSE));

			ircsprintf(fbuf, NETWORK_GLINNED_NAME": %s", reason);
			exit_client(acptr, acptr, &me, fbuf);
            i--;
        }
    }

    host = get_userban_host(ban, fbuf, 512);

    if(temporary_gline_time)
    {	
		sendto_realops("%s added a temporary %d sec(s) "NETWORK_GLINE_NAME" for [%s@%s] [%s]",
			parv[0], temporary_gline_time, user, host, reason);
		sendto_serv_butone(sptr, ":%s GLINE %d %s@%s :%s",
			parv[0], temporary_gline_time_seconds, user, host, reason);
		return 0;
    }

    sendto_realops("%s added a permanent gline for [%s@%s] [%s]",
		   parv[0], user, host, reason);
	sendto_serv_butone(sptr, ":%s GLINE %d %s@%s :%s",
		parv[0], temporary_gline_time_seconds, user, host, reason);

    return 0;
}

int m_ungline(aClient *cptr, aClient *sptr, int parc, char *parv[]) 
{
    struct userBan *ban;
	char *user, *host;
    
    if (!IsAnOper(sptr))
    {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
    }
	
	if(parc<2) 
    {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0],"UNGLINE");
		return 0;
    }

    if ((host = strchr(parv[1], '@')) || *parv[1] == '*') {
	if (host) {
		user = parv[1];
		*(host++) = '\0';
	} else {
		user = "*";
		host = parv[1];
	}
    } else {
		sendto_one(sptr, ":%s NOTICE %s :Invalid parameters",me.name, parv[0]);
		return 0;
    }
    if ((user[0] == '*') && (user[1] == '\0') && 
	(host[0] == '*') && (host[1] == '\0'))  {
	sendto_one(sptr, ":%s NOTICE %s :Cannot ungline everyone",me.name, parv[0]);
	return 0;
    }

    ban = make_hostbased_ban(user, host);
	if(ban) {
		struct userBan *oban;
		ban->flags |= (UBAN_GLINE|UBAN_TEMPORARY);

		if((oban = find_userban_exact(ban, UBAN_GLINE))) {
			char tmp[512];
			host = get_userban_host(oban, tmp, 512);
			remove_userban(oban);
			userban_free(oban);
			userban_free(ban);
			
			if (MyClient(sptr))
				sendto_one(sptr, ":%s NOTICE %s :G-line for [%s@%s] is removed",
				me.name, parv[0], user, host);
			sendto_ops("%s has removed the G-line for: [%s@%s] (%d matches)",
				parv[0], user, host, 1);

			sendto_serv_butone(cptr, ":%s UNGLINE %s@%s", sptr->name, user, host);
			return 0;
		}
		userban_free(ban);
	}
	if (MyClient(sptr))
		sendto_one(sptr, ":%s NOTICE %s :No g-line for %s@%s was found",
	           me.name, parv[0], user, host);
    return 0;
}
